@@ -115,6 +115,9 @@ enum {
 #define MAP_STACK 0x20000
 #endif
 
+#ifndef MADV_FREE
+#define MADV_FREE 8
+#endif
 #ifndef MADV_MERGEABLE
 #define MADV_MERGEABLE 12
 #endif
@@ -165,6 +168,10 @@ enum {
 #define PF_IB          AF_IB
 #endif
 
+#ifndef PF_MPLS
+#define PF_MPLS		28
+#endif
+
 #ifndef PF_CAN
 #define PF_CAN		29
 #endif
@@ -214,6 +221,10 @@ enum {
 #define AF_VSOCK PF_VSOCK
 #endif
 
+#ifndef PF_KCM
+#define PF_KCM		41
+#endif
+
 #ifndef NFC_SOCKPROTO_RAW
 #define NFC_SOCKPROTO_RAW	0
 #endif
@@ -225,6 +236,10 @@ enum {
 #define MSG_WAITFORONE	0x10000
 #endif
 
+#ifndef MSG_BATCH
+#define MSG_BATCH 0x40000
+#endif
+
 #ifndef MSG_CMSG_CLOEXEC
 #define MSG_CMSG_CLOEXEC	0x40000000
 #endif
@@ -265,6 +280,27 @@ enum {
 #ifndef NETLINK_CAP_ACK
 #define NETLINK_CAP_ACK 10
 #endif
+#ifndef NETLINK_SOCK_DIAG
+#define NETLINK_SOCK_DIAG 4
+#endif
+#ifndef RTNLGRP_DCB
+#define RTNLGRP_DCB 23
+#endif
+#ifndef RTNLGRP_IPV4_NETCONF
+#define RTNLGRP_IPV4_NETCONF 24
+#endif
+#ifndef RTNLGRP_IPV6_NETCONF
+#define RTNLGRP_IPV6_NETCONF 25
+#endif
+#ifndef RTNLGRP_MDB
+#define RTNLGRP_MDB 26
+#endif
+#ifndef RTNLGRP_MPLS_ROUTE
+#define RTNLGRP_MPLS_ROUTE 27
+#endif
+#ifndef RTNLGRP_NSID
+#define RTNLGRP_NSID 28
+#endif
 
 /* linux/prctl.h */
 #ifndef PR_MCE_KILL_GET
@@ -436,6 +472,19 @@ enum kcmp_type {
 #define SO_INCOMING_CPU		49
 #endif
 
+#ifndef SO_ATTACH_BPF
+#define SO_ATTACH_BPF		50
+#endif
+
+#ifndef SO_ATTACH_REUSEPORT_CBPF
+#define SO_ATTACH_REUSEPORT_CBPF 51
+#define SO_ATTACH_REUSEPORT_EBPF 52
+#endif
+
+#ifndef SO_CNX_ADVICE
+#define SO_CNX_ADVICE 53
+#endif
+
 /* linux/tcp.h */
 #ifndef TCP_COOKIE_TRANSACTIONS
 #define TCP_COOKIE_TRANSACTIONS	15
@@ -490,6 +539,10 @@ enum kcmp_type {
 #define TCP_SAVED_SYN		28
 #endif
 
+#ifndef TCP_REPAIR_WINDOW
+#define TCP_REPAIR_WINDOW	29
+#endif
+
 /* linux/if_packet.h */
 #ifndef PACKET_VNET_HDR
 #define PACKET_VNET_HDR		15
@@ -1018,13 +1071,20 @@ struct kvm_get_htab_fd {
 #define SWAP_FLAG_DISCARD 0x10000
 #endif
 
-/* unistd.h */
+/* linux/fs.h */
 #ifndef SEEK_DATA
 #define SEEK_DATA 3
 #endif
 #ifndef SEEK_HOLE
 #define SEEK_HOLE 4
 #endif
+#ifndef RWF_HIPRI
+#define RWF_HIPRI 0x00000001 /* high priority request, poll if possible */
+#endif
+#ifndef RWF_DSYNC
+#define RWF_DSYNC 0x00000002 /* per-IO O_DSYNC */
+#define RWF_SYNC  0x00000004 /* per-IO O_SYNC */
+#endif
 
 /* sys/epoll.h */
 #ifndef EPOLLWAKEUP
@@ -1046,3 +1106,27 @@ struct kvm_get_htab_fd {
 #ifndef POLL_BUSY_LOOP
 #define POLL_BUSY_LOOP 0x8000
 #endif
+
+/* linux/nvme_ioctl.h */
+#ifndef NVME_IOCTL_RESET
+#define NVME_IOCTL_RESET _IO('N', 0x44)
+#endif
+
+/* linux/shm.h */
+#ifndef SHM_HUGE_SHIFT
+#define SHM_HUGE_SHIFT  26
+#endif
+#ifndef SHM_HUGE_2MB
+#define SHM_HUGE_2MB	(21 << SHM_HUGE_SHIFT)
+#define SHM_HUGE_1GB	(30 << SHM_HUGE_SHIFT)
+#endif
+
+/* bits/shm.h */
+#ifndef SHM_NORESERVE
+# define SHM_NORESERVE 010000
+#endif
+
+/* netfilter/ipset/ipset.h */
+#ifndef SO_IP_SET
+#define SO_IP_SET 83
+#endif
