@@ -6,6 +6,8 @@
 #include "files.h"
 #include "ioctls.h"
 #include "log.h"
+#include "random.h"
+#include "sanitise.h"
 #include "shm.h"
 #include "utils.h"	// ARRAY_SIZE
 
@@ -30,31 +32,53 @@ void register_ioctl_group(const struct ioctl_group *grp)
 	++grps_cnt;
 }
 
+static const struct ioctl_group * match_ioctl(int fd, struct stat *stbuf, int matchcount)
+{
+	int i;
+
+retry:
+	for (i = 0; i < grps_cnt; ++i) {
+		if (grps[i]->fd_test) {
+			if (grps[i]->fd_test(fd, stbuf) == 0) {
+				/* if this is the only match, just do it. */
+				if (matchcount == 1) {
+					return grps[i];
+				} else {
+					if (RAND_BOOL())
+						return grps[i];
+				}
+			}
+		}
+	}
+
+	// If we get here we failed the RAND_BOOL too many times.
+	goto retry;
+}
+
 const struct ioctl_group *find_ioctl_group(int fd)
 {
 	const char *devname;
 	struct stat stbuf;
 	int i;
 	size_t j;
+	int matchcount = 0;
 
 	if (fstat(fd, &stbuf) < 0)
 		return NULL;
 
-	if (stbuf.st_rdev == 0)
-		return NULL;
-
-	devname = map_dev(stbuf.st_rdev, stbuf.st_mode);
-	if (!devname)
-		return NULL;
-
-	for (i=0; i < grps_cnt; ++i) {
+	/* Find out if >1 ioctl with an fd_test matches this fd type. */
+	for (i = 0; i < grps_cnt; ++i) {
 		if (grps[i]->fd_test) {
 			if (grps[i]->fd_test(fd, &stbuf) == 0)
-				return grps[i];
-			else
-				continue;
+				matchcount++;
 		}
+	}
 
+	if (matchcount > 0)
+		return match_ioctl(fd, &stbuf, matchcount);
+
+	/* We don't have an fd_test, so try matching on type & devname */
+	for (i = 0; i < grps_cnt; ++i) {
 		switch (grps[i]->devtype) {
 		case DEV_MISC:
 			/* fall through. misc devices are char devices. */
@@ -69,6 +93,13 @@ const struct ioctl_group *find_ioctl_group(int fd)
 		default: break;
 		}
 
+		if (stbuf.st_rdev == 0)
+			return NULL;
+
+		devname = map_dev(stbuf.st_rdev, stbuf.st_mode);
+		if (!devname)
+			return NULL;
+
 		for (j=0; j < grps[i]->devs_cnt; ++j)
 			if (strcmp(devname, grps[i]->devs[j]) == 0)
 				return grps[i];
@@ -82,16 +113,34 @@ const struct ioctl_group *get_random_ioctl_group(void)
 	if (grps_cnt == 0)
 		return NULL;
 
-	return grps[rand() % grps_cnt];
+	return grps[rnd() % grps_cnt];
+}
+
+static unsigned long random_ioctl_arg(void)
+{
+	if (RAND_BOOL()) {
+		return (unsigned long) rand64();
+	} else {
+		void *page;
+
+		page = get_writable_address(page_size);
+		generate_random_page(page);
+
+		return (unsigned long) page;
+	}
 }
 
 void pick_random_ioctl(const struct ioctl_group *grp, struct syscallrecord *rec)
 {
 	int ioctlnr;
 
-	ioctlnr = rand() % grp->ioctls_cnt;
+	ioctlnr = rnd() % grp->ioctls_cnt;
 
 	rec->a2 = grp->ioctls[ioctlnr].request;
+	rec->a3 = random_ioctl_arg();
+	rec->a4 = random_ioctl_arg();
+	rec->a5 = random_ioctl_arg();
+	rec->a6 = random_ioctl_arg();
 }
 
 void dump_ioctls(void)
