@@ -17,7 +17,8 @@ static void sanitise_mincore(struct syscallrecord *rec)
 
 	map = common_set_mmap_ptr_len();
 
-	len = map->size + (page_size - 1) / page_size;
+	len = min(GB(1), map->size);
+	len = len + (page_size - 1) / page_size;
 
 	rec->a3 = (unsigned long) zmalloc(len);	// FIXME: LEAK
 }
