@@ -245,16 +245,20 @@ static int parse_generic(int pmu, const char *value,
 				if (value[ptr]==0) break;
 				if (value[ptr]==',') break;
 				if (! ( ((value[ptr]>='0') && (value[ptr]<='9'))
-                   			|| ((value[ptr]>='a') && (value[ptr]<='f'))) ) {
+					|| ((value[ptr]>='a') && (value[ptr]<='f'))
+					|| ((value[ptr]>='A') && (value[ptr]<='F'))) ) {
 					outputerr("Unexpected char %c\n", value[ptr]);
 				}
 				temp*=base;
 				if ((value[ptr]>='0') && (value[ptr]<='9')) {
 					temp+=value[ptr]-'0';
 				}
-				else {
+				else if ((value[ptr]>='a') && (value[ptr]<='f')) {
 					temp+=(value[ptr]-'a')+10;
 				}
+				else {
+					temp+=(value[ptr]-'A')+10;
+				}
 				i++;
 				ptr++;
 			}
@@ -483,17 +487,17 @@ static long long random_sysfs_config(__u32 *type,
 		return rand64();
 	}
 
-	i=rand()%num_pmus;
+	i=rnd()%num_pmus;
 
 	*type=pmus[i].type;
 
-	switch(rand()%3) {
+	switch(rnd()%3) {
 		/* Random by Format */
 		case 0:
 			if (pmus[i].num_formats==0) goto out;
 			for(j=0;j<pmus[i].num_formats;j++) {
 				/* 50% chance of having field set */
-				if (rand()%2) {
+				if (rnd()%2) {
 					if (pmus[i].formats[j].field==FIELD_CONFIG) {
 						c|=(rand64()&pmus[i].formats[j].mask);
 					} else if (pmus[i].formats[j].field==FIELD_CONFIG1) {
@@ -509,7 +513,7 @@ static long long random_sysfs_config(__u32 *type,
 		/* Random by generic event */
 		case 1:
 			if (pmus[i].num_generic_events==0) goto out;
-			j=rand()%pmus[i].num_generic_events;
+			j=rnd()%pmus[i].num_generic_events;
 			c=pmus[i].generic_events[j].config;
 			c1=pmus[i].generic_events[j].config1;
 			c2=pmus[i].generic_events[j].config2;
@@ -527,8 +531,8 @@ static long long random_sysfs_config(__u32 *type,
 	*config2=c2;
 	return c;
 out:
-	*config1=rand()%64;
-	return rand()%64;
+	*config1=rnd()%64;
+	return rnd()%64;
 }
 
 /* arbitrary high number unlikely to be used by perf_event */
@@ -540,7 +544,7 @@ static long long random_cache_config(void)
 
 	int cache_id, hw_cache_op_id, hw_cache_op_result_id;
 
-	switch (rand() % 8) {
+	switch (rnd() % 8) {
 	case 0:
 		cache_id = PERF_COUNT_HW_CACHE_L1D;
 		break;
@@ -570,7 +574,7 @@ static long long random_cache_config(void)
 		break;
 	}
 
-	switch (rand() % 4) {
+	switch (rnd() % 4) {
 	case 0:
 		hw_cache_op_id = PERF_COUNT_HW_CACHE_OP_READ;
 		break;
@@ -588,7 +592,7 @@ static long long random_cache_config(void)
 		break;
 	}
 
-	switch (rand() % 3) {
+	switch (rnd() % 3) {
 	case 0:
 		hw_cache_op_result_id = PERF_COUNT_HW_CACHE_RESULT_ACCESS;
 		break;
@@ -611,7 +615,7 @@ static int random_event_type(void)
 
 	int type=0;
 
-	switch (rand() % 8) {
+	switch (rnd() % 8) {
 	case 0:
 		type = PERF_TYPE_HARDWARE;
 		break;
@@ -650,7 +654,7 @@ static long long random_event_config(__u32 *event_type,
 
 	switch (*event_type) {
 	case PERF_TYPE_HARDWARE:
-		switch (rand() % 11) {
+		switch (rnd() % 11) {
 		case 0:
 			config = PERF_COUNT_HW_CPU_CYCLES;
 			break;
@@ -689,7 +693,7 @@ static long long random_event_config(__u32 *event_type,
 		}
 		break;
 	case PERF_TYPE_SOFTWARE:
-		switch (rand() % 11) {
+		switch (rnd() % 12) {
 		case 0:
 			config = PERF_COUNT_SW_CPU_CLOCK;
 			break;
@@ -721,6 +725,9 @@ static long long random_event_config(__u32 *event_type,
 			config = PERF_COUNT_SW_DUMMY;
 			break;
 		case 10:
+			config = PERF_COUNT_SW_BPF_OUTPUT;
+			break;
+		case 11:
 			config = rand64();
 			break;
 		default:
@@ -731,10 +738,10 @@ static long long random_event_config(__u32 *event_type,
 		/* Actual values to use can be found under */
 		/* debugfs tracing/events/?*?/?*?/id       */
 		/* usually a small < 4096 number           */
-		switch(rand()%2) {
+		switch(rnd()%2) {
 		case 0:
 			/* Try a value < 4096 */
-			config = rand()&0xfff;
+			config = rnd()&0xfff;
 			break;
 		case 1:
 			config = rand64();
@@ -778,7 +785,7 @@ static long long random_event_config(__u32 *event_type,
 static void setup_breakpoints(struct perf_event_attr *attr)
 {
 
-	switch (rand() % 6) {
+	switch (rnd() % 6) {
 	case 0:
 		attr->bp_type = HW_BREAKPOINT_EMPTY;
 		break;
@@ -806,7 +813,7 @@ static void setup_breakpoints(struct perf_event_attr *attr)
 	/* or a valid mem location for R/W/RW             */
 	attr->bp_addr = (long)get_address();
 
-	switch (rand() % 5) {
+	switch (rnd() % 5) {
 	case 0:
 		attr->bp_len = HW_BREAKPOINT_LEN_1;
 		break;
@@ -901,7 +908,7 @@ static int random_attr_size(void) {
 
 	int size=0;
 
-	switch(rand() % 10) {
+	switch(rnd() % 10) {
 	case 0:	size = PERF_ATTR_SIZE_VER0;
 		break;
 	case 1: size = PERF_ATTR_SIZE_VER1;
@@ -956,6 +963,10 @@ static long long random_branch_sample_type(void)
 		branch_sample |= PERF_SAMPLE_BRANCH_COND;
 	if (RAND_BOOL())
 		branch_sample |= PERF_SAMPLE_BRANCH_CALL_STACK;
+	if (RAND_BOOL())
+		branch_sample |= PERF_SAMPLE_BRANCH_IND_JUMP;
+	if (RAND_BOOL())
+		branch_sample |= PERF_SAMPLE_BRANCH_CALL;
 
 	/* Transactional Memory Types */
 	if (RAND_BOOL())
@@ -1002,7 +1013,7 @@ static void create_mostly_valid_counting_event(struct perf_event_attr *attr,
 	attr->enable_on_exec = RAND_BOOL();
 	attr->task = RAND_BOOL();
 	attr->watermark = RAND_BOOL();
-	attr->precise_ip = rand() % 4;	// two bits
+	attr->precise_ip = rnd() % 4;	// two bits
 	attr->mmap_data = RAND_BOOL();
 	attr->sample_id_all = RAND_BOOL();
 	attr->exclude_host = RAND_BOOL();
@@ -1012,6 +1023,8 @@ static void create_mostly_valid_counting_event(struct perf_event_attr *attr,
 	attr->mmap2 = RAND_BOOL();
 	attr->comm_exec = RAND_BOOL();
 	attr->use_clockid = RAND_BOOL();
+	attr->context_switch = RAND_BOOL();
+	attr->write_backward = RAND_BOOL();
 
 	/* wakeup events not relevant */
 
@@ -1028,6 +1041,10 @@ static void create_mostly_valid_counting_event(struct perf_event_attr *attr,
 	/* sample_regs_user not relevant if not sampling */
 
 	/* sample_stack_user not relevant if not sampling */
+
+	/* aux_watermark not relevant if not sampling */
+
+	/* sample_max_stack not relevant if not sampling */
 }
 
 static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
@@ -1066,7 +1083,7 @@ static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
 	attr->enable_on_exec = RAND_BOOL();
 	attr->task = RAND_BOOL();
 	attr->watermark = RAND_BOOL();
-	attr->precise_ip = rand() % 4;	// two bits
+	attr->precise_ip = rnd() % 4;	// two bits
 	attr->mmap_data = RAND_BOOL();
 	attr->sample_id_all = RAND_BOOL();
 	attr->exclude_host = RAND_BOOL();
@@ -1076,6 +1093,8 @@ static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
 	attr->mmap2 = RAND_BOOL();
 	attr->comm_exec = RAND_BOOL();
 	attr->use_clockid = RAND_BOOL();
+	attr->context_switch = RAND_BOOL();
+	attr->write_backward = RAND_BOOL();
 
 	attr->wakeup_events = rand32();
 
@@ -1092,10 +1111,10 @@ static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
 	/* sample_regs_user is a bitmask of CPU registers to record.     */
 	/* The values come from arch/ARCH/include/uapi/asm/perf_regs.h   */
 	/* Most architectures have fewer than 64 registers...            */
-	switch(rand()%3) {
-		case 0:		attr->sample_regs_user = rand()%16;
+	switch(rnd()%3) {
+		case 0:		attr->sample_regs_user = rnd()%16;
 				break;
-		case 1:		attr->sample_regs_user = rand()%64;
+		case 1:		attr->sample_regs_user = rnd()%64;
 				break;
 		case 2:		attr->sample_regs_user = rand64();
 				break;
@@ -1108,7 +1127,7 @@ static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
 	attr->sample_stack_user = rand32();
 
 	if (attr->use_clockid) {
-		switch(rand()%6) {
+		switch(rnd()%6) {
 			case 0:	attr->clockid = CLOCK_MONOTONIC;
 				break;
 			case 1: attr->clockid = CLOCK_MONOTONIC_RAW;
@@ -1120,11 +1139,13 @@ static void create_mostly_valid_sampling_event(struct perf_event_attr *attr,
 			/* Most possible values < 32 */
 			case 4: attr->clockid = RAND_BYTE();
 				break;
-			case 5:	attr->clockid = rand();
+			case 5:	attr->clockid = rnd();
 				break;
 		}
 	}
 
+	attr->aux_watermark = rand32();
+	attr->sample_max_stack = rand32();
 }
 
 
@@ -1189,7 +1210,7 @@ static void create_random_event(struct perf_event_attr *attr)
 	attr->enable_on_exec = RAND_BOOL();
 	attr->task = RAND_BOOL();
 	attr->watermark = RAND_BOOL();
-	attr->precise_ip = rand() % 4;
+	attr->precise_ip = rnd() % 4;
 	attr->mmap_data = RAND_BOOL();
 	attr->sample_id_all = RAND_BOOL();
 	attr->exclude_host = RAND_BOOL();
@@ -1244,15 +1265,14 @@ void sanitise_perf_event_open(struct syscallrecord *rec)
 	/* should usually be -1 or another perf_event fd         */
 	/* Anything but -1 unlikely to work unless the other pid */
 	/* was properly set up to be a group master              */
-	switch (rand() % 3) {
+	switch (rnd() % 3) {
 	case 0:
 		rec->a4 = -1;
 		group_leader = 1;
 		break;
 	case 1:
 		/* Try to get a previous random perf_event_open() fd  */
-		/* It's unclear whether get_random_fd() would do this */
-		rec->a4 = rand() % 1024;
+		rec->a4 = get_rand_perf_fd();
 		break;
 	case 2:
 		/* Rely on ARG_FD */
@@ -1286,7 +1306,7 @@ void sanitise_perf_event_open(struct syscallrecord *rec)
 		/* a file descriptor from /dev/cgroup       */
 		pid = get_random_fd();
 	} else {
-		switch(rand() % 4) {
+		switch(rnd() % 4) {
 		case 0:	/* use current thread */
 			pid = 0;
 			break;
@@ -1307,7 +1327,7 @@ void sanitise_perf_event_open(struct syscallrecord *rec)
 	rec->a2 = pid;
 
 	/* set up attr structure */
-	switch (rand() % 4) {
+	switch (rnd() % 4) {
 	case 0:
 		create_mostly_valid_counting_event(attr,group_leader);
 		break;
@@ -1330,6 +1350,10 @@ static void post_perf_event_open(struct syscallrecord *rec)
 	freeptr(&rec->a1);
 }
 
+static unsigned long perf_event_open_flags[] = {
+	PERF_FLAG_FD_NO_GROUP, PERF_FLAG_FD_OUTPUT, PERF_FLAG_PID_CGROUP,
+};
+
 struct syscallentry syscall_perf_event_open = {
 	.name = "perf_event_open",
 	.num_args = 5,
@@ -1343,12 +1367,7 @@ struct syscallentry syscall_perf_event_open = {
 	.arg4type = ARG_FD,
 	.arg5name = "flags",
 	.arg5type = ARG_LIST,
-	.arg5list = {
-		.num = 3,
-		.values = {
-			PERF_FLAG_FD_NO_GROUP, PERF_FLAG_FD_OUTPUT, PERF_FLAG_PID_CGROUP
-		},
-	},
+	.arg5list = ARGLIST(perf_event_open_flags),
 	.sanitise = sanitise_perf_event_open,
 	.post = post_perf_event_open,
 	.init = init_pmus,
