@@ -3,7 +3,7 @@
  */
 #include "sanitise.h"
 
-struct syscall syscall_sched_get_priority_max = {
+struct syscallentry syscall_sched_get_priority_max = {
 	.name = "sched_get_priority_max",
 	.num_args = 1,
 	.arg1name = "policy",
