@@ -377,5 +377,13 @@ extern struct syscallentry syscall_kexec_file_load;
 extern struct syscallentry syscall_bpf;
 extern struct syscallentry syscall_execveat;
 extern struct syscallentry syscall_userfaultfd;
+extern struct syscallentry syscall_membarrier;
+extern struct syscallentry syscall_mlock2;
+extern struct syscallentry syscall_copy_file_range;
+extern struct syscallentry syscall_preadv2;
+extern struct syscallentry syscall_pwritev2;
+extern struct syscallentry syscall_pkey_mprotect;
+extern struct syscallentry syscall_pkey_alloc;
+extern struct syscallentry syscall_pkey_free;
 
 unsigned int random_fcntl_setfl_flags(void);
