@@ -6,6 +6,8 @@
 #include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
+#include "child.h"
+#include "debug.h"
 #include "log.h"
 #include "params.h"
 #include "trinity.h"
@@ -29,24 +31,25 @@ void dump_uids(void)
 		uid, gid, euid, egid, suid, sgid);
 }
 
-void drop_privs(void)
+bool drop_privs(struct childdata *child)
 {
 	if (setresgid(nobody_gid, nobody_gid, nobody_gid) < 0) {
-		outputerr("Error setting nobody gid (%s)\n", strerror(errno));
+		output(0, "Error setting nobody gid (%s)\n", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
 	if (setgroups(0, NULL) == -1) {
-		outputerr("Error dropping supplemental groups (%s)\n", strerror(errno));
-		exit(EXIT_FAILURE);
+		;
 	}
 
 	if (setresuid(nobody_uid, nobody_uid, nobody_uid) < 0) {
-		outputerr("Error setting nobody uid (%s)\n", strerror(errno));
+		output(0, "Error setting nobody uid (%s)\n", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
-	outputstd("set uid to %u and gid to %d (nobody)\n", nobody_uid, nobody_gid);
+//	debugf("set uid to %u and gid to %d (nobody)\n", nobody_uid, nobody_gid);
+	child->dropped_privs = TRUE;
+	return TRUE;
 }
 
 void init_uids(void)
@@ -93,6 +96,11 @@ void do_uid0_check(void)
 		}
 	}
 
+	if (clowntown == TRUE) {
+		printf("THIS CLOWN GOES TO 11.\n");
+		return;
+	}
+
 	outputstd("ctrl-c now unless you really know what you are doing.\n");
 	for (i = 10; i > 0; i--) {
 		outputstd("Continuing in %u seconds.\r", i);
